//+------------------------------------------------------------------+
//|                                           MartingaleEnvelope.mq4 |
//|                   Copyleft  2010, http://davidke20.blogspot.com |
//|                                               rb26team@yahoo.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, http://davidke20.blogspot.com"
#property link      "rb26team@yahoo.com"

#define NONE 0
#define UP   1
#define DN   2

extern double  InitLotsize    = 0.01;
extern double  InitTP         = 10;
extern double  InitSL         = 10;
extern bool    Auto           = false;
extern double  Multiplier     = 2.0;
extern int     MaxLossOrders  = 3;
extern int     TPincrement    = 5;
extern int     MagicNumber    = 877218;
extern int     Slippage       = 3;
extern int     ATR_Period     = 62;
extern int     ATR_TF         = 60;
extern int     Envelope       = 12;
extern int     Env_type       = 0;
extern double  Deviation      = 0.10;
extern int     Shift          = 0;

datetime       Timestamp      = 0;
int            Signal         = 0,
               LossCount      = 0,
               Ticket         = 0;
double         SL             = 0,
               TP             = 0,
               Lotsize        = 0;

int init()
{
   if(Digits==3 || Digits==5)
   {
      InitTP=InitTP*10;
      InitSL=InitSL*10;
      TPincrement=TPincrement*10;
      Slippage=Slippage*10;
   }
   return(0);
}


int start()
{
double         MinLots = NormalizeDouble((MarketInfo(Symbol(), MODE_MINLOT)),2);
double         MaxLots = NormalizeDouble((MarketInfo(Symbol(), MODE_MAXLOT)),2);

   if(InitTP<10 || InitSL<10)
   {
      Comment("Please redefine a better take profit and/or stop loss");
      return(0);
   }

   Comment("Martingale trader\n\n",
            "Current lot size : ",Lotsize,
            "\nMaximum lot size allowed : ",MaxLots,
            "\nMinimum lot size allowed : ",MinLots);

   if(Timestamp != iTime(NULL,0,0))
   {
      Timestamp = iTime(NULL,0,0);     // Mark this bar as been finished processing
      if(!CurrentTrade())              // No active open order
      {
         if(TradingSignal() == UP)
         {
         CheckTradeHistory();
            Ticket=OrderSend(Symbol(),OP_BUY,Lotsize,Ask,Slippage,Ask-SL*Point,Ask+TP*Point,"Martingale",MagicNumber,0,Green);
            if(Ticket > 0)
            {
               if(OrderSelect(Ticket,SELECT_BY_TICKET,MODE_TRADES)) Print("BUY order opened : ",OrderOpenPrice());
            }
            else if(!IsTesting())Print("Error opening BUY order : ",GetLastError()); 
         }

         if(TradingSignal() == DN)
         {
         CheckTradeHistory();
            Ticket=OrderSend(Symbol(),OP_SELL,Lotsize,Bid,Slippage,Bid+SL*Point,Bid-TP*Point,"Martingale",MagicNumber,0,Red);
            if(Ticket > 0)
            {
               if(OrderSelect(Ticket,SELECT_BY_TICKET,MODE_TRADES)) Print("SELL order opened : ",OrderOpenPrice());
            }
            else if(!IsTesting())Print("Error opening SELL order : ",GetLastError()); 
         }
      }
   }
   return(0);
}


int deinit()
{
   return(0);
}


bool CurrentTrade()
{
   for(int cnt=0;cnt<OrdersTotal();cnt++)             // Do an inventory on current trade status
   {
      if(OrderSelect(cnt,SELECT_BY_POS))
      {
         if (OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber)
            return(true);
      }
   }
   return(false);
}


int CheckTPSL()
{
   if(Auto)
   InitSL=iATR(NULL,ATR_TF,ATR_Period,Shift)/Point;
   InitTP=iATR(NULL,ATR_TF,ATR_Period,Shift)/Point;
   if(Digits==3||Digits==5)
   {
      if(InitSL<100){InitSL=100;}
      if(InitTP<100){InitTP=100;}
   }
   else
   {
      if(InitSL<10){InitSL=10;}
      if(InitTP<10){InitTP=10;}
   }
}

bool CheckTradeHistory()
{
   if(Auto)CheckTPSL();
   datetime LastTradeTime   = 0;
   double   LastOrderProfit = 0;
   double   MinLots = NormalizeDouble((MarketInfo(Symbol(), MODE_MINLOT)),2);
   double   MaxLots = NormalizeDouble((MarketInfo(Symbol(), MODE_MAXLOT)),2);

   for (int cnt=0; cnt<OrdersHistoryTotal(); cnt++)   // Do an inventory on history trades status
   { 
      if (OrderSelect(cnt, SELECT_BY_POS,MODE_HISTORY))
      {
         if (OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber)
         {
            if (OrderOpenTime() > LastTradeTime)
            {
               LastTradeTime   = OrderOpenTime();
               LastOrderProfit = OrderProfit();
            }     
         }
      }
   }

   if(LossCount>=MaxLossOrders-1)
   {  LossCount=0;
      LastOrderProfit=1;
   }

   if(LastOrderProfit < 0)
   {
      LossCount = LossCount+1;
      TP       += TPincrement;
      Lotsize  *= Multiplier;
   }
   else
   {
      TP       = InitTP;
      SL       = InitSL;
      Lotsize  = InitLotsize;
   }
      if(Lotsize>=MaxLots) Lotsize = MaxLots;
      if(Lotsize<=MinLots) Lotsize = MinLots;
      Lotsize  = NormalizeDouble(Lotsize,2);
   return(0);
}

int TradingSignal()
{
   Signal = NONE;
   double   dPriceHigh,  dPriceLow, PriceHigh, PriceLow;

     dPriceHigh   = iEnvelopes(NULL,0,Envelope,Env_type,0,PRICE_CLOSE,Deviation,MODE_UPPER,Shift+1);
     dPriceLow    = iEnvelopes(NULL,0,Envelope,Env_type,0,PRICE_CLOSE,Deviation,MODE_LOWER,Shift+1);
     PriceHigh    = iEnvelopes(NULL,0,Envelope,Env_type,0,PRICE_CLOSE,Deviation,MODE_UPPER,Shift);
     PriceLow     = iEnvelopes(NULL,0,Envelope,Env_type,0,PRICE_CLOSE,Deviation,MODE_LOWER,Shift);

   if(iLow(NULL,0,1)<dPriceLow && iClose(NULL,0,1)>dPriceLow && iHigh(NULL,0,0)<PriceHigh)
   {Signal = UP;}
   if(iHigh(NULL,0,1)>dPriceHigh && iClose(NULL,0,1)<dPriceHigh && iLow(NULL,0,0)>PriceLow)
   {Signal = DN;}

   return(Signal);
}







